@@ -9,6 +9,7 @@ from aiopulse.errors import (
     NotRunningException,
     InvalidResponseException,
 )
+from aiopulse.const import UpdateType
 
 __all__ = [
     "Hub",
@@ -19,8 +20,9 @@ __all__ = [
     "NotConnectedException",
     "NotRunningException",
     "InvalidResponseException",
+    "UpdateType",
 ]
-__version__ = "0.3.4"
+__version__ = "0.4.0"
 __author__ = "Alan Murray"
 
 _LOGGER = logging.getLogger(__name__)
